"use strict";
var tinymce = tinymce || {};

/*
 * TinyMCE plugin for inserting media from FotoWare.
 *
 * Opens a child window and then uses the postMessage API to
 * receive the image to insert into the editor.
 *
 * Settings: (Settings can be done from TinyMceModule config in Episerver solution)
 *   fotowareBaseUrl (required) - the base URL to FotoWare (e.g. https://my.fotoware.cloud), normally this is the fotowareUiUrl in fotoware.client.config
 *   useIframeProxy (optional) - if set to true a window with iframe will be used for compability with IE11
 *   debugLog (optional) - if set to true logs debug messages
 */

tinymce.PluginManager.add('fotoware', function (editor, url) {

    // polyfill Promise if needed
    if (typeof Promise !== 'function') {
        tinymce.ScriptLoader.loadScripts([url + '/lib/es6-promise.min.js'],
            function () {
                window.Promise = ES6Promise;
            });
    }

    var hostcontent = getHashParameterByName("context");
    hostcontent = (hostcontent !== null) ? hostcontent.replace("epi.cms.contentdata:///", "") : "";

    // reference to the window
    var fwWindow;
    var windowUrl;

    // url to iframe proxy page
    const iframeProxyPage = url + '/fotoware.html';

    // url to error page
    const errorPage = url + '/error.html';

    // url to FotoWare button icon
    const fotowareIcon = url + '/favicon.ico';

    // url to edit button icon
    const cropIcon = url + '/crop.svg';

    // true if FotoWare window is open and communication is in progress,
    // false otherwise
    var fwWindowCommunicating = false;

    // use event when window is closing
    var eventDispatcher = new tinymce.util.EventDispatcher();

    // reference to the ping timeout so we can clear it
    // when window closes
    var pingTimer;

    // polyfill for location.origin
    if (typeof location.origin === 'undefined') {
        location.origin = location.protocol + '//' + location.host;
    }

    // origin of fotoware / iframe proxy
    var targetOrigin;

    const logEnabled = editor.settings.debugLog;

    // features of the FotoWare window
    const fwWindowFeatures = logEnabled ? '': 'width=1200,height=800,resizable=yes,scrollbars';

    /*
     * Adds the button to insert FotoWare media and handles
     * the insert action
     */
    editor.addButton('fotoware-insert-media',
    {
        title: 'Insert',
        image: fotowareIcon,
        tooltip: 'Insert FotoWare media',
        onclick: function () {
            verifyConfig()
                .then(function() {
                    // Focus tiny (to trigger save and publish events)
                    editor.focus();

                    // Open FotoWare
                    openFotoWareWindow('insert');
                })
                .then(waitForWindowToClose)
                .then(function (media) { addMediaToEditor(media); })
                .catch(function (error) { handleError(error); });
        }
    });

    /*
     * Adds the button that edits FotoWare media and handles
     * the edit action 
     */
    //editor.addButton('fotoware-edit-media',
    //    {
    //        title: 'Edit',
    //        image: cropIcon,
    //        tooltip: 'Edit selected FotoWare media',
    //        onclick: function () {
    //            verifyConfig()
    //                .then(function () { openFotoWareWindow('edit'); })
    //                .then(waitForWindowToClose)
    //                .then(function (media) { addMediaToEditor(media); })
    //                .catch(function (error) { handleError(error); });
    //        },
    //        onpostrender: monitorNodeChange
    //    });

    /*
     * Edit button is only enabled when we have
     * an img selected.
     */
    //function monitorNodeChange() {
    //    var btn = this;
    //    editor.on('NodeChange',
    //        function (e) {
    //            btn.disabled(!e.element || !e.element.nodeName || e.element.nodeName.toLowerCase() !== "img");
    //        });
    //}

    /*
     * Verifies all required editor configuration exist
     */
    function verifyConfig() {
        if (!editor.settings.fotowareBaseUrl) {
            return Promise.reject(Error('Missing required editor configuration: fotowareBaseUrl'));
        }

        if (editor.settings.useIframeProxy) {
            targetOrigin = location.origin;
        } else {
            targetOrigin = editor.settings.fotowareBaseUrl;
        }

        // Make sure target origin does not have trailing slash
        if (targetOrigin.slice(-1) === '/') {
            targetOrigin = targetOrigin.slice(0, -1);
        }

        return Promise.resolve();
    }

    /*
     * Opens FotoWare window in 'insert' or 'edit' mode
     */
    function openFotoWareWindow(mode) {
        try {
            windowUrl = createUrl(mode);
            const windowName = 'FwTinyMce_' + editor.id;

            logEnabled && console.info('Opening window with name: ' + windowName);

            fwWindow = window.open(windowUrl, windowName, fwWindowFeatures);

            if (window.addEventListener) {
                fwWindow.opener.addEventListener('message', messageEvent, false);
            } else if (window.attachEvent) {
                window.attachEvent('onmessage', messageEvent);
            }

            return Promise.resolve();
        } catch (e) {
            return Promise.reject(Error('Error opening FotoWare window: ' + e.message));
        }
    }

    /*
     * Waits for event when external window has closed
     */
    function waitForWindowToClose() {
        return new Promise(function (resolve) {
            eventDispatcher.on('fwWinClosed', function (e) {
                eventDispatcher.off('fwWinClosed');
                resolve(e.media);
            });
        });
    }


    /*
     * Handle errors nicely
     */
    function handleError(error) {
        editor.windowManager.open({
            title: 'FotoWare plugin error',
            url: errorPage,
            width: 700,
            height: 400
        },
            {
                errorMessage: error
            });
    }

    /*
     * Create absolute URL with required parameters to insert or edit media
     */
    function createUrl(mode) {
        // common params for all requests
        var params = {
            fotowareBaseUrl: editor.settings.fotowareBaseUrl,
            version: editor.settings.version
        };

        if (mode === 'edit') {
            // include mediaUrl for edit
            const selectedImage = tinymce.activeEditor.selection.getNode();

            // src for poster image for video (to edit video)
            const videoSrc = selectedImage.getAttribute("data-mce-p-poster");

            var mediaUrl = "";

            if (videoSrc) {
                mediaUrl = videoSrc;
            } else {
                mediaUrl = selectedImage.getAttribute("src");
            }

            params.mediaUrl = mediaUrl;
        }

        const query = Object.keys(params)
            .map(function (k) { return encodeURIComponent(k) + '=' + encodeURIComponent(params[k]); })
            .join('&');

        var baseUrl;

        // check if we should call FotoWare directly
        // or use iframe proxy
        if (editor.settings.useIframeProxy) {
            baseUrl = iframeProxyPage;
        } else {
            baseUrl = editor.settings.fotowareBaseUrl;
        }

        logEnabled && console.info('FotoWare URL: ' + baseUrl + '?' + query);
        return baseUrl + '?' + query;
    }
   
    /*
     * Handler for incoming postMessages
     */
    function messageEvent(message) {
        // Ignore messages not for us.
        if (message.origin !== targetOrigin) {
            logEnabled && console.info('Message received to another origin: (' + message.origin + ' != ' + targetOrigin + '): ' + message.data);
            return;
        }

        logEnabled && console.log(message.data, event);

        //ta hand om loggedinSuccess, skicka om till fotowareApi.url + Math.random()
        if (message.data == "loggedinSuccess") {
            fwWindow.location = windowUrl + "&" + Math.random();
            return;
        }

        switch (message.data.event) {
        case 'assetSelected':
                logEnabled && console.log("An asset was selected: " + message.data.asset.href);
                eventDispatcher.fire('fwWinClosed', { media: message.data.asset.href });
            break;
        case 'selectionWidgetCancel':
            logEnabled && console.log("cancel, closing window");
            closeWindow();
            break;

        default:
        }
    }

    /*
     * Inserts media (if any) into the editor
     */
    function addMediaToEditor(media) {
        if (media) {
            
            ajax(editor.settings.fotowaremodulepage + "authentication/fetch?hostcontent="
                + hostcontent
                + "&image=" + media,
                function (xhttp) {
                    //debugger 
                    //success
                    logEnabled && console.log("image found", xhttp.responseText);

                    editor.insertContent(xhttp.responseText);
                    closeWindow(); 
                },
                function (xhttp) {
                    closeWindow(); 
                    handleError(xhttp.statusText);
                });
        }
    }

    /*
     * Closes and cleans up the window and handlers
     */
    function closeWindow() {
        try {
            fwWindow.opener.removeEventListener('message', messageEvent);
            fwWindow.close();
        } catch (e) {

        } finally {
            fwWindowCommunicating = false;
            eventDispatcher.fire('fwWinClosed', { media: null });
        }
    }

    function ajax(url, successFunc, errFunc) {
        var xhttp;
        xhttp = new XMLHttpRequest();
        xhttp.onreadystatechange = function () {
            if (this.readyState == 4 && this.status == 200) {
                successFunc(this); 
            }
            if (this.readyState == 4 && this.status != 200) {
                errFunc(this);
            }
        };
        xhttp.open("GET", url, true);
        xhttp.send();
    }

    function getHashParameterByName(name) {
        const url = window.location.hash;
        name = name.replace(/[\[\]]/g, '\\$&');
        const regex = new RegExp('[?#&]' + name + '(=([^&#]*)|&|#|$)');
        const results = regex.exec(url);

        if (!results) {
            return null;
        }

        if (!results[2]) {
            return '';
        }

        return decodeURIComponent(results[2].replace(/\+/g, ' '));
    }

    /*
     * TinyMCE metadata shows up in the ? menu
     */
    return {
        getMetadata: function () {
            return {
                name: 'FotoWare',
                url: 'https://www.fotoware.com'
            };
        }
    };
});